package sun.java2d.loops;

import java.lang.ref.WeakReference;
import sun.java2d.pipe.SpanIterator;
import java.awt.image.Raster;
import java.awt.CompositeContext;
import java.awt.image.WritableRaster;
import java.awt.RenderingHints;
import sun.java2d.pipe.Region;
import java.awt.Composite;
import sun.java2d.SurfaceData;

public class Blit extends GraphicsPrimitive
{
    public static final String methodSignature;
    public static final int primTypeID;
    private static RenderCache blitcache;
    
    public static Blit locate(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        return (Blit)GraphicsPrimitiveMgr.locate(Blit.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public static Blit getFromCache(SurfaceType obj, CompositeType obj2, SurfaceType obj3) {
        Object value = Blit.blitcache.get(obj, obj2, obj3);
        if (value != null) {
            return (Blit)value;
        }
        Blit locate = locate(obj, obj2, obj3);
        if (locate == null) {
            System.out.println("blit loop not found for:");
            System.out.println("src:  " + obj);
            System.out.println("comp: " + obj2);
            System.out.println("dst:  " + obj3);
            return locate;
        }
        Blit.blitcache.put(obj, obj2, obj3, locate);
        return locate;
    }
    
    protected Blit(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        super(Blit.methodSignature, Blit.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public Blit(long n, SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        super(n, Blit.methodSignature, Blit.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public native void Blit(SurfaceData p0, SurfaceData p1, Composite p2, Region p3, int p4, int p5, int p6, int p7, int p8, int p9);
    
    @Override
    public GraphicsPrimitive makePrimitive(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        if (compositeType.isDerivedFrom(CompositeType.Xor)) {
            Blit.GeneralXorBlit generalXorBlit = new GeneralXorBlit(surfaceType, compositeType, surfaceType2);
            this.setupGeneralBinaryOp(generalXorBlit);
            return generalXorBlit;
        }
        if (compositeType.isDerivedFrom(CompositeType.AnyAlpha)) {
            return new GeneralMaskBlit(surfaceType, compositeType, surfaceType2);
        }
        return Blit.AnyBlit.instance;
    }
    
    @Override
    public GraphicsPrimitive traceWrap() {
        return new Blit.TraceBlit(this);
    }
    
    static {
        methodSignature = "Blit(...)".toString();
        primTypeID = GraphicsPrimitive.makePrimTypeID();
        Blit.blitcache = new RenderCache(20);
        GraphicsPrimitiveMgr.registerGeneral(new Blit(null, null, null));
    }
    
    private static class AnyBlit extends Blit
    {
        public static Blit.AnyBlit instance;
        
        public AnyBlit() {
            super(SurfaceType.Any, CompositeType.Any, SurfaceType.Any);
        }
        
        @Override
        public void Blit(SurfaceData surfaceData, SurfaceData surfaceData2, Composite composite, Region instanceXYWH, int n, int n2, int x, int y, int w, int h) {
            CompositeContext context = composite.createContext(surfaceData.getColorModel(), surfaceData2.getColorModel(), new RenderingHints(null));
            Raster raster = surfaceData.getRaster(n, n2, w, h);
            WritableRaster writableRaster = (WritableRaster)surfaceData2.getRaster(x, y, w, h);
            if (instanceXYWH == null) {
                instanceXYWH = Region.getInstanceXYWH(x, y, w, h);
            }
            int[] bbox = { x, y, x + w, y + h };
            SpanIterator spanIterator = instanceXYWH.getSpanIterator(bbox);
            n -= x;
            n2 -= y;
            while (spanIterator.nextSpan(bbox)) {
                int n3 = bbox[2] - bbox[0];
                int n4 = bbox[3] - bbox[1];
                Raster child = raster.createChild(n + bbox[0], n2 + bbox[1], n3, n4, 0, 0, (int[])null);
                WritableRaster writableChild = writableRaster.createWritableChild(bbox[0], bbox[1], n3, n4, 0, 0, (int[])null);
                context.compose(child, writableChild, writableChild);
            }
            context.dispose();
        }
        
        static {
            Blit.AnyBlit.instance = new Blit.AnyBlit();
        }
    }
    
    private static class GeneralMaskBlit extends Blit
    {
        MaskBlit performop;
        
        public GeneralMaskBlit(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
            super(surfaceType, compositeType, surfaceType2);
            this.performop = MaskBlit.locate(surfaceType, compositeType, surfaceType2);
        }
        
        @Override
        public void Blit(SurfaceData surfaceData, SurfaceData surfaceData2, Composite composite, Region region, int n, int n2, int n3, int n4, int n5, int n6) {
            this.performop.MaskBlit(surfaceData, surfaceData2, composite, region, n, n2, n3, n4, n5, n6, null, 0, 0);
        }
    }
    
    private static class GeneralXorBlit extends Blit implements GraphicsPrimitive.GeneralBinaryOp
    {
        Blit convertsrc;
        Blit convertdst;
        Blit performop;
        Blit convertresult;
        WeakReference srcTmp;
        WeakReference dstTmp;
        
        public GeneralXorBlit(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
            super(surfaceType, compositeType, surfaceType2);
        }
        
        @Override
        public void setPrimitives(Blit convertsrc, Blit convertdst, GraphicsPrimitive graphicsPrimitive, Blit convertresult) {
            this.convertsrc = convertsrc;
            this.convertdst = convertdst;
            this.performop = (Blit)graphicsPrimitive;
            this.convertresult = convertresult;
        }
        
        @Override
        public synchronized void Blit(SurfaceData convert, SurfaceData convert2, Composite composite, Region region, int n, int n2, int n3, int n4, int n5, int n6) {
            if (this.convertsrc == null) {
            }
            else {
                SurfaceData surfaceData = null;
                if (this.srcTmp != null) {
                    surfaceData = (SurfaceData)this.srcTmp.get();
                }
                convert = GraphicsPrimitive.convertFrom(this.convertsrc, convert, n, n2, n5, n6, surfaceData);
                n = 0;
                n2 = 0;
                if (convert != surfaceData) {
                    this.srcTmp = new WeakReference(convert);
                }
            }
            if (this.convertdst == null) {
            }
            else {
                SurfaceData surfaceData2 = null;
                if (this.dstTmp != null) {
                    surfaceData2 = (SurfaceData)this.dstTmp.get();
                }
                convert2 = GraphicsPrimitive.convertFrom(this.convertdst, convert2, n3, n4, n5, n6, surfaceData2);
                n3 = 0;
                n4 = 0;
                region = null;
                if (convert2 != surfaceData2) {
                    this.dstTmp = new WeakReference(convert2);
                }
            }
            this.performop.Blit(convert, convert2, composite, region, n, n2, n3, n4, n5, n6);
            if (this.convertresult != null) {
                GraphicsPrimitive.convertTo(this.convertresult, convert2, convert2, region, n3, n4, n5, n6);
            }
        }
    }
    
    private static class TraceBlit extends Blit
    {
        Blit target;
        
        public TraceBlit(Blit target) {
            super(target.getSourceType(), target.getCompositeType(), target.getDestType());
            this.target = target;
        }
        
        @Override
        public GraphicsPrimitive traceWrap() {
            return this;
        }
        
        @Override
        public void Blit(SurfaceData surfaceData, SurfaceData surfaceData2, Composite composite, Region region, int n, int n2, int n3, int n4, int n5, int n6) {
            GraphicsPrimitive.tracePrimitive(this.target);
            this.target.Blit(surfaceData, surfaceData2, composite, region, n, n2, n3, n4, n5, n6);
        }
    }
}
